
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/* set information about this example */
shared_lcrzoex_infos lcrzoex_000458=
{ /*reference*/     "lcrzoex", 458, 
  /*version*/       1, 0, 
  /*title*/         "RzoBox : display information about Host/Ip/Ethernet", 
  /*french title*/  "RzoBox : affiche des infos sur Host/Ip/Ethernet", 
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "addr language",
  /*french usage*/  "adresse langue", 
  /*usage example*/ "www.microsoft.com 2",
  /*fr. usage ex.*/ "www.microsoft.com 2",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000458_main(int argc, char *argv[]);
int lcrzoex_000458_hs(lcrzo_consths hs, lcrzo_int16 language);
int lcrzoex_000458_ipl(lcrzo_ipl ipl, lcrzo_int16 language);
int lcrzoex_000458_etha(lcrzo_constetha etha, lcrzo_int16 language);

/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000458_main(int argc, char *argv[])
{ lcrzo_int16 language;
  lcrzo_hs hs;
  lcrzo_ipl ipl;
  lcrzo_etha etha;
  int ret;

  lcrzo_init();
  if ( argc<3 )
  { /* print usage */
    lcrzo_epr(shared_lcrzoex_infos_print(lcrzoex_000458, argv[0]));
    return(LCRZOEX_ERR_MISC_BADTOOLUSAGE);
  }
  lcrzo_epr(lcrzo_int16_init_text(argv[2], &language));

  /* set the current language */
  lcrzo_epr(lcrzo_global_set_language(language));

  /* guess the address type */
  ret=lcrzo_etha_init_eths(argv[1], etha);
  if (ret==LCRZO_ERR_OK)
  { lcrzo_epr(lcrzoex_000458_etha(etha, language));
  }
  else
  { ret=lcrzo_ipl_init_ips(argv[1], &ipl);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_epr(lcrzoex_000458_ipl(ipl, language));
    }
    else
    { ret=lcrzo_hs_init_hs(argv[1], hs);
      if (ret==LCRZO_ERR_OK)
      { lcrzo_epr(lcrzoex_000458_hs(hs, language));
      }
      else
      { if ( language==LCRZO_GLOBAL_LANG_FR )
        { printf("'%s' n'est pas une adresse IP, une adresse ", argv[1]);
          printf("Ethernet ou un nom de machine\n");
	}
        else
        { printf("'%s' is not an IP address, an Ethernet address", argv[1]);
          printf(" or a hostname\n");
	}
      }
    }
  }

  lcrzo_close();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_hs_print(lcrzo_consths hs, lcrzo_int16 language);
int lcrzoex_000458_hs_print(lcrzo_consths hs, lcrzo_int16 language)
{
  if (language == LCRZO_GLOBAL_LANG_FR) {
    lcrzo_er(lcrzo_hs_print("Nom de machine : ", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_hs_print("Hostname : ", hs, "\n"));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_hs_print_unr(lcrzo_int16 language);
int lcrzoex_000458_hs_print_unr(lcrzo_int16 language)
{
  if (language == LCRZO_GLOBAL_LANG_FR) {
    printf("Nom de machine : non resolu\n");
  } else {
    printf("Hostname : unresolved\n");
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_ipl_print(lcrzo_ipl ipl, lcrzo_int16 language);
int lcrzoex_000458_ipl_print(lcrzo_ipl ipl, lcrzo_int16 language)
{
  if ( language==LCRZO_GLOBAL_LANG_FR ) {
    lcrzo_er(lcrzo_ipl_print("Adresse IP : ", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_ipl_print("IP address : ", ipl, "\n"));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_ipl_print_unr(lcrzo_int16 language);
int lcrzoex_000458_ipl_print_unr(lcrzo_int16 language)
{
  if ( language==LCRZO_GLOBAL_LANG_FR ) {
    printf("Adresse IP : non resolue\n");
  } else {
    printf("IP address : unresolved\n");
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_ipl_print_rout(lcrzo_ipl ipl, lcrzo_int16 language);
int lcrzoex_000458_ipl_print_rout(lcrzo_ipl ipl, lcrzo_int16 language)
{ lcrzo_device device;
  lcrzo_etha srcetha, dstetha;
  lcrzo_ipl srcipl, routeripl;
  int ret;

  ret = lcrzo_conf_route_to_host(ipl, device, srcetha, dstetha,
				 &srcipl, &routeripl);
  if (ret == LCRZO_ERR_OK) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("Pour sniffer le flux de cette machine, ");
      printf("il faut utiliser le device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("Pour construire un paquet pour cette machine, ");
      printf("il faut utiliser les adresses :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      lcrzo_er(lcrzo_etha_print(" - Ethernet destination : ", dstetha, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("Le flux traverse le routeur : ",
				 routeripl, "\n"));
      }
    } else {
      printf("To sniff flow of this computer, ");
      printf("the following device has to be used :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("To construct a packet for this computer, ");
      printf("following addresses have to be used :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      lcrzo_er(lcrzo_etha_print(" - Ethernet destination : ", dstetha, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("The flow goes through the router : ",
				 routeripl, "\n"));
      }
    }
  } else if (ret == LCRZO_ERR_OKROUTELOCALDEV) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("Cette adresse correspond au device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
    } else {
      printf("This address corresponds to device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
    }
  } else if (ret == LCRZO_ERR_OKROUTEHOSTUNREACH) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("Pour sniffer le flux de cette machine, ");
      printf("il faut utiliser le device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("Pour construire un paquet pour cette machine, ");
      printf("il faut utiliser les adresses :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      printf(" - Ethernet destination : non resolue (sur le LAN)\n");
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
    } else {
      printf("To sniff flow of this computer, ");
      printf("the following device has to be used :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("To construct a packet for this computer, ");
      printf("following addresses have to be used :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      printf(" - Ethernet destination : unresolved (on the LAN)\n");
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
    }
  } else if (ret == LCRZO_ERR_OKROUTEGWUNREACH) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("Pour sniffer le flux de cette machine, ");
      printf("il faut utiliser le device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("Pour construire un paquet pour cette machine, ");
      printf("il faut utiliser les adresses :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      printf(" - Ethernet destination : non resolue ");
      lcrzo_er(lcrzo_ipl_print("(adresse de ", routeripl, ")\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("Le flux traverse le routeur : ",
				 routeripl, "\n"));
      }
    } else {
      printf("To sniff flow of this computer, ");
      printf("the following device has to be used :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("To construct a packet for this computer, ");
      printf("following addresses have to be used :\n");
      lcrzo_er(lcrzo_etha_print(" - Ethernet source : ", srcetha, "\n"));
      printf(" - Ethernet destination : unresolved ");
      lcrzo_er(lcrzo_ipl_print("(address of ", routeripl, ")\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("The flow goes through the router : ",
				 routeripl, "\n"));
      }
    }
  } else if (ret == LCRZO_ERR_OKROUTENOTETHER) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("Pour sniffer le flux de cette machine, ");
      printf("il faut utiliser le device :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("Pour construire un paquet pour cette machine, ");
      printf("il faut utiliser les adresses :\n");
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("Le flux traverse le routeur : ",
				 routeripl, "\n"));
      }
    } else {
      printf("To sniff flow of this computer, ");
      printf("the following device has to be used :\n");
      lcrzo_er(lcrzo_device_print("   ", device, "\n"));
      printf("To construct a packet for this computer, ");
      printf("following addresses have to be used :\n");
      lcrzo_er(lcrzo_ipl_print(" - IP source : ", srcipl, "\n"));
      lcrzo_er(lcrzo_ipl_print(" - IP destination : ", ipl, "\n"));
      if (routeripl != 0) {
	lcrzo_er(lcrzo_ipl_print("The flow goes through the router : ",
				 routeripl, "\n"));
      }
    }
  } else if (ret == LCRZO_ERR_OKROUTENOTFOUND) {
    if ( language==LCRZO_GLOBAL_LANG_FR ) {
      printf("La table de routage ne contient pas ");
      printf("de route vers cette machine.\n");
    } else {
      printf("The routing table doesn't contain a route to this host.\n");
    }
    ret=lcrzo_etha_init_ipl(ipl, dstetha);
    if (ret == LCRZO_ERR_OK) {
      if ( language==LCRZO_GLOBAL_LANG_FR ) {
	lcrzo_er(lcrzo_etha_print("Adresse Ethernet : ", dstetha, "\n"));
      } else {
	lcrzo_er(lcrzo_etha_print("Ethernet address : ", dstetha, "\n"));
      }
    }
  } else {
    return(ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_etha_print(lcrzo_constetha etha, lcrzo_int16 language);
int lcrzoex_000458_etha_print(lcrzo_constetha etha, lcrzo_int16 language)
{
  if (language == LCRZO_GLOBAL_LANG_FR) {
    lcrzo_er(lcrzo_etha_print("Adresse Ethernet : ", etha, "\n"));
  } else {
    lcrzo_er(lcrzo_etha_print("Ethernet address : ", etha, "\n"));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_hs(lcrzo_consths hs, lcrzo_int16 language)
{ lcrzo_ipl ipl;
  int ret;

  lcrzo_er(lcrzoex_000458_hs_print(hs, language));

  ret = lcrzo_ipl_init_hs(hs, &ipl);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzoex_000458_ipl_print_unr(language));
    /* leave here */
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzoex_000458_ipl_print(ipl, language));

  lcrzo_er(lcrzoex_000458_ipl_print_rout(ipl, language));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_ipl(lcrzo_ipl ipl, lcrzo_int16 language)
{ lcrzo_hs hs;
  int ret;

  lcrzo_er(lcrzoex_000458_ipl_print(ipl, language));

  ret = lcrzo_hs_init_ipl(ipl, hs);
  if (ret == LCRZO_ERR_OK) {
    lcrzo_er(lcrzoex_000458_hs_print(hs, language));
  } else {
    lcrzo_er(lcrzoex_000458_hs_print_unr(language));
  }

  lcrzo_er(lcrzoex_000458_ipl_print_rout(ipl, language));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000458_etha(lcrzo_constetha etha, lcrzo_int16 language)
{ lcrzo_ipl ipl;
  lcrzo_hs hs;
  int ret;

  lcrzo_er(lcrzoex_000458_etha_print(etha, language));

  ret = lcrzo_ipl_init_etha(etha, &ipl);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzoex_000458_ipl_print_unr(language));
    /* leave here */
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzoex_000458_ipl_print(ipl, language));

  ret = lcrzo_hs_init_ipl(ipl, hs);
  if (ret == LCRZO_ERR_OK) {
    lcrzo_er(lcrzoex_000458_hs_print(hs, language));
  } else {
    lcrzo_er(lcrzoex_000458_hs_print_unr(language));
  }

  lcrzo_er(lcrzoex_000458_ipl_print_rout(ipl, language));

  return (LCRZO_ERR_OK);
}

